@@ -141,6 +141,37 @@ oil_prototype_to_string (OilPrototype *proto)
   return oil_string_free (string, FALSE);
 }
 
+/**
+ * oil_prototype_to_arg_string:
+ * @proto: the OilPrototype
+ *
+ * Converts a prototype into the corresponding C argument list.
+ *
+ * Returns: a string that should be freed using free() when it is no
+ * longer needed.
+ */
+char *
+oil_prototype_to_arg_string (OilPrototype *proto)
+{
+  OilString *string;
+  OilParameter *param;
+  int i;
+
+  string = oil_string_new ("");
+
+  for(i=0;i<proto->n_params;i++){
+    param = proto->params + i;
+
+    oil_string_append (string, param->parameter_name);
+
+    if(i<proto->n_params-1){
+      oil_string_append (string, ", ");
+    }
+  }
+
+  return oil_string_free (string, FALSE);
+}
+
 /**
  * oil_prototype_from_string:
  * @s: a string
@@ -416,16 +447,16 @@ static const char *oil_type_names_2 [] = {
   "double"
 };
 static const char *oil_type_names_3 [] = {
-  "type_s8",
-  "type_u8",
-  "type_s16",
-  "type_u16",
-  "type_s32",
-  "type_u32",
-  "type_s64",
-  "type_u64",
-  "type_f32",
-  "type_f64"
+  "oil_type_s8",
+  "oil_type_u8",
+  "oil_type_s16",
+  "oil_type_u16",
+  "oil_type_s32",
+  "oil_type_u32",
+  "oil_type_s64",
+  "oil_type_u64",
+  "oil_type_f32",
+  "oil_type_f64"
 };
 #define ARRAY_LENGTH(x) (sizeof(x)/sizeof(x[0]))
 
@@ -720,4 +751,13 @@ oil_param_from_string (OilParameter *p, char *s)
   return 1;
 }
 
+void *oil_param_get_source_data (OilParameter *param)
+{
+  uint8_t *ptr;
+ 
+  ptr = param->src_data;
+  ptr += param->test_header;
+
+  return ptr;
+}
 
